Option Explicit On 
Option Strict On

Imports System.Runtime.InteropServices

Public Module modMain
    <DllImport("KERNEL32.DLL")> _
    Public Sub CopyMemory( _
        ByVal Destination As System.Int32, _
        ByVal Source As System.Int32, _
        ByVal Length As System.Int32 _
    )
        ' <native implementation>
    End Sub

    Private Declare Sub CopyMemoryByRef Lib _
      "kernel32.dll" Alias "RtlMoveMemory" ( _
        ByRef Destination As System.Int32, _
        ByRef Source As System.Int32, _
        ByVal Length As System.Int32 _
    )

    Public Sub Main()
        Console.WriteLine("*** Beispiel 1")

        Dim strSource As String = "Hallo Welt!"
        CConsoleHelper.WriteLineStringVariable( _
          "strSource", strSource)
        Dim strDest As String = "ActiveVB.de"
        CConsoleHelper.WriteLineStringVariable( _
          "strDest", strDest)

        ' Fixieren der Variablen im Speicher, sodass 
        ' sie der Garbage Collector nicht mehr
        ' verschieben kann.
        Dim hgcSource As GCHandle = _
          GCHandle.Alloc(strSource, GCHandleType.Pinned)
        Dim hhcDest As GCHandle = _
          GCHandle.Alloc(strDest, GCHandleType.Pinned)

        Console.WriteLine("Kopieren von strSource ber strDest...")

        ' Hier werden die entsprechenden Bytes kopiert. 
        ' Strings bestehen aus Zeichen, die je
        ' 2 Byte lang sind, aus diesem Grund werden 
        ' strSource.Length * 2 Bytes kopiert.
        CopyMemory( _
            hhcDest.AddrOfPinnedObject.ToInt32(), _
            hgcSource.AddrOfPinnedObject.ToInt32(), _
            strSource.Length * 2 _
        )

        CConsoleHelper.WriteLineStringVariable( _
          "strSource", strSource)
        CConsoleHelper.WriteLineStringVariable( _
          "strDest", strDest)

        ' Dem GC wieder erlauben, die Variablen im 
        ' Speicher zu verschieben.
        hgcSource.Free()
        hhcDest.Free()

        Console.WriteLine("*** Beispiel 2")

        Dim x As Integer = 99
        Dim y As Integer = 144
        Console.WriteLine("x = {0}", x)
        Console.WriteLine("y = {0}", y)
        Console.WriteLine("Kopieren des Wertes von y ber x...")

        ' Kopieren eines Int32, d.h. 4 Bytes von y nach x.
        CopyMemoryByRef(x, y, 4)

        Console.WriteLine("x = {0}", x)
        Console.WriteLine("y = {0}", y)

        Console.ReadLine()
    End Sub

    Private Class CConsoleHelper
        Public Shared Sub WriteLineStringVariable( _
          ByRef strName As String, _
          ByRef strValue As String)
            Console.WriteLine("{0} = ""{1}""", _
              strName, strValue)
        End Sub
    End Class
End Module